from rapyuta_io.utils.utils import prepend_bearer_to_auth_token, create_auth_header, get_api_response_data
from rapyuta_io.utils.rest_client import HttpMethod
from rapyuta_io.utils import RestClient, to_objdict
from rapyuta_io.clients.static_route import StaticRoute


class CoreAPIClient:

    STATIC_ROUTE_PATH = '/api/staticroute'

    def __init__(self, auth_token, project, core_api_host):
        self._core_api_host = core_api_host
        self._auth_token = prepend_bearer_to_auth_token(auth_token)
        self._project = project

    def _add_auth_token_to_routes(self, routes):
        for route in routes:
            setattr(route, '_core_api_host', self._core_api_host)
            setattr(route, '_auth_token', self._auth_token)
            setattr(route, '_project', self._project)

    def _get_all_static_routes(self):
        url = self._core_api_host + self.STATIC_ROUTE_PATH + '/list'
        headers = create_auth_header(self._auth_token, self._project)
        response = RestClient(url).headers(headers).execute()
        return get_api_response_data(response, parse_full=True)

    def _get_static_route(self, route_guid):
        url = "{}{}/{}/get".format(self._core_api_host, self.STATIC_ROUTE_PATH, route_guid)
        headers = create_auth_header(self._auth_token, self._project)
        response = RestClient(url).headers(headers).execute()
        return get_api_response_data(response, parse_full=True)

    def _create_static_route(self, url_prefix):
        url = self._core_api_host + self.STATIC_ROUTE_PATH + '/create'
        headers = create_auth_header(self._auth_token, self._project)
        payload = {"urlPrefix": url_prefix}
        response = RestClient(url).method(HttpMethod.POST).headers(headers).execute(payload)
        return get_api_response_data(response, parse_full=True)

    def get_all_static_routes(self):
        static_routes = []
        data = self._get_all_static_routes()
        for route in data:
            static_route = StaticRoute(to_objdict(route))
            static_routes.append(static_route)
        self._add_auth_token_to_routes(static_routes)
        return static_routes

    def get_static_route(self, route_guid):
        data = self._get_static_route(route_guid)
        route = StaticRoute(to_objdict(data))
        self._add_auth_token_to_routes([route])
        return route

    def create_static_route(self, name):
        data = self._create_static_route(name)
        route = StaticRoute(to_objdict(data))
        self._add_auth_token_to_routes([route])
        return route

    def get_static_route_by_name(self, name):
        routes = self._get_all_static_routes()
        for route in routes:
            if route['urlPrefix'] == name:
                return StaticRoute(to_objdict(route))
        return None

