from rapyuta_io.clients.deployment import _poll_till_ready
from rapyuta_io.utils import ObjDict
from rapyuta_io.utils import RestClient
from rapyuta_io.utils import to_objdict
from rapyuta_io.utils.rest_client import HttpMethod
from rapyuta_io.utils.utils import create_auth_header, get_api_response_data


class RoutedNetwork(ObjDict):
    """
    RoutedNetwork represents Routed Network

    :ivar name: Name of RoutedNetwork.
    :ivar runtime: Runtime of RoutedNetwork enum :py:class:`~rapyuta_io.clients.package.Runtime`.
    :ivar rosDistro: ROSDistro of RoutedNetwork enum :py:class:`~rapyuta_io.clients.package.ROSDistro`.
    :ivar shared: Whether the network can be shared (bool).
    :ivar phase: Deployment phase :py:class:`~rapyuta_io.clients.deployment.DeploymentPhaseConstants`
    :ivar status: Deployment status :py:class:`~rapyuta_io.clients.deployment.DeploymentStatus`
    :ivar error_code: Deployment errors
    """

    ROUTED_NETWORK_PATH = 'routednetwork'

    def __init__(self, *args, **kwargs):
        super(ObjDict, self).__init__(*args, **kwargs)

    def poll_routed_network_till_ready(self, retry_count=120, sleep_interval=5):
        _poll_till_ready(self, retry_count, sleep_interval)
        return self

    def get_status(self):
        url = '{}/{}/{}'.format(self._host, self.ROUTED_NETWORK_PATH, self.guid)
        headers = create_auth_header(self._auth_token, self._project)
        response = RestClient(url).method(HttpMethod.GET).headers(headers).execute()
        response = get_api_response_data(response, parse_full=True)
        routed_network = RoutedNetwork(to_objdict(response))
        internal_deployment_status = routed_network.internalDeploymentStatus
        internal_deployment_status.errors = routed_network.get_error_code()
        return internal_deployment_status

    def delete(self):
        url = '{}/{}/{}'.format(self._host, self.ROUTED_NETWORK_PATH, self.guid)
        headers = create_auth_header(self._auth_token, self._project)
        response = RestClient(url).method(HttpMethod.DELETE).headers(headers).execute()
        get_api_response_data(response, parse_full=True)
        self.clear()
        return True

    def get_error_code(self):
        return self.internalDeploymentStatus.error_code if hasattr(self.internalDeploymentStatus, 'error_code') else []
