# encoding: utf-8
import json
from time import sleep

import enum
import requests
from requests.exceptions import RequestException

from rapyuta_io.utils import APIError

DEFAULT_RETRY_COUNT = 4
WAIT_TIME_IN_SEC = 1


class HttpMethod(enum.Enum):

    def __str__(self):
        return str(self.value)

    GET = 'GET'
    POST = 'POST'
    PUT = 'PUT'
    PATCH = 'PATCH'
    DELETE = 'DELETE'
    OPTIONS = 'OPTIONS'


class RestClient:
    def __init__(self, url):
        self._url = url
        self._retry_limit = DEFAULT_RETRY_COUNT
        self._retry_count = 0
        self._method = HttpMethod.GET.value
        self._headers = {}
        self._query_params = {}

    def url(self, url):
        self._url = url
        return self

    def method(self, http_method):
        self._method = http_method.value
        return self

    def headers(self, headers):
        self._headers = headers
        return self

    def retry(self, retry_limit):
        if type(retry_limit) is int:
            self._retry_limit = retry_limit
        return self

    def query_param(self, query_param):
        self._query_params = query_param
        return self

    def execute(self, payload=None):
        while True:
            try:
                response = requests.request(method=self._method, url=self._url,
                                            headers=self._headers, params=self._query_params,
                                            json=payload if payload else None)

                return response
            except RequestException as err:
                if self._retry_count >= self._retry_limit:
                    raise APIError("Error occurred for URL {}: {}".format(self._url, err.message))
                sleep(WAIT_TIME_IN_SEC)
                self._retry_count = self._retry_count + 1
